package com.ningdatech.pmapi.common.util;

import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.GenericResult;
import com.ningdatech.pmapi.todocenter.bean.entity.WorkNoticeInfo;
import com.ningdatech.zwdd.client.ZwddClient;
import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.*;

/**
 * 异步发送工作通知工具类
 *
 * @author CMM
 * @since 2023/02/23 13:50
 */
@Slf4j
public class SendWorkNoticeUtil {

    private SendWorkNoticeUtil(){

    }

    public static void sendWorkNotice(ZwddClient zwddClient, WorkNoticeInfo workNoticeInfo, String msg){
        // 初始化线程池
        ThreadPoolExecutor threadPool = new ThreadPoolExecutor(10, 20,
                60, TimeUnit.SECONDS, new ArrayBlockingQueue(20), new ThreadPoolExecutor.AbortPolicy());

        // 将发送工作通知交给异步任务Future
        CompletableFuture<String> future = CompletableFuture.supplyAsync(() -> {
            // 调用浙政钉的接口发送工作通知
            long startTime = System.currentTimeMillis();
            GenericResult<Void> result =
                    zwddClient.sendWorkNotice(workNoticeInfo.getReceiverUserId(), workNoticeInfo.getBizMsgId(), msg);
            String resultMsg = result.getMsg();
            if (resultMsg.equals("success")) {
                log.info("异步任务执行完成, " + workNoticeInfo.getBizMsgId() + " 当前线程：" + Thread.currentThread().getName());
                long endTime = System.currentTimeMillis();
                log.info("方法执行完成返回，耗时：" + (endTime - startTime));
            }else {
                return "发送工作通知失败!";
            }
            return "发送工作通知成功!";
        }, threadPool);
        String s;
        try {
            s = future.get();
        }  catch (Exception e) {
            throw new BizException("获取异步线程处理结果失败!");
        }
        threadPool.shutdown();
        while (threadPool.isTerminated()) {
            log.info(s);
            break;
        }
    }
}
