package com.ningdatech.pmapi.expert.manage;

import com.ningdatech.pmapi.expert.assembler.ExpertInfoCmdAssembler;
import com.ningdatech.pmapi.expert.constant.ExpertAccountStatusEnum;
import com.ningdatech.pmapi.expert.constant.ExpertUserInfoStepEnum;
import com.ningdatech.pmapi.expert.entity.ExpertUserFullInfo;
import com.ningdatech.pmapi.expert.helper.ExpertManageHelper;
import com.ningdatech.pmapi.expert.model.ExpertBasicInfo;
import com.ningdatech.pmapi.expert.model.ExpertEduInfo;
import com.ningdatech.pmapi.expert.model.ExpertJobInfo;
import com.ningdatech.pmapi.expert.model.ExpertProfessionalInfo;
import com.ningdatech.pmapi.expert.model.cmd.ExpertFullInfoSaveCmd;
import com.ningdatech.pmapi.expert.model.req.ExpertUserBasicInfoSubmitRequest;
import com.ningdatech.pmapi.expert.service.ExpertInfoService;
import com.ningdatech.pmapi.expert.service.IExpertUserFullInfoService;
import com.ningdatech.pmapi.meta.model.ExpertRegionInfo;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

/**
 * @author liuxinxin
 * @date 2023/2/23 上午8:55
 */

@Component
@RequiredArgsConstructor
public class ExpertManage {

    private final ExpertManageHelper expertManageHelper;

    private final IExpertUserFullInfoService iExpertUserFullInfoService;

    private final ExpertInfoService expertInfoService;


    /**
     * 填写基本信息,只有专家自己可用
     *
     * @param request
     */
    public void expertBasicInfoSubmit(ExpertUserBasicInfoSubmitRequest request) {
        // 用户id
        Long userId = LoginUserUtil.getUserId();
        ExpertBasicInfo basicInfo = request.getBasicInfo();
        // 校验区域编码合法性 校验履职意向编码合法性
        ExpertRegionInfo expertRegionInfo = basicInfo.getExpertRegionInfo();
        expertManageHelper.expertRegionInfoCheck(expertRegionInfo);
        List<ExpertRegionInfo> expertIntentionWorkRegions = basicInfo.getExpertIntentionWorkRegions();
        for (ExpertRegionInfo expertIntentionWorkRegion : expertIntentionWorkRegions) {
            expertManageHelper.expertRegionInfoCheck(expertIntentionWorkRegion);
        }
        ExpertEduInfo eduInfo = request.getEduInfo();
        ExpertJobInfo jobInfo = request.getJobInfo();
        ExpertProfessionalInfo professionalInfo = request.getProfessionalInfo();
        // 校验标签字段
        expertManageHelper.tagFieldCheck(professionalInfo, basicInfo);
        // 校验字典字段
        expertManageHelper.dictionaryFieldCheck(basicInfo, eduInfo, jobInfo);
        // 判断专家提交状态，判断是否可以进行此操作
        ExpertUserFullInfo expertUserFullInfo = iExpertUserFullInfoService.getByUserId(userId);

        boolean submitBasicInfoStatusEnable = Objects.isNull(expertUserFullInfo)
                || (ExpertAccountStatusEnum.APPLYING.getKey().equals(expertUserFullInfo.getExpertAccountStatus())
                && !ExpertUserInfoStepEnum.EVIDENCE_HAS_BEEN_SUBMITTED.getKey().equals(expertUserFullInfo.getUserInfoStep()));
        if (submitBasicInfoStatusEnable) {
            // 新建 保存
            ExpertFullInfoSaveCmd expertFullInfoSaveCmd = ExpertInfoCmdAssembler
                    .buildExpertFullInfoSaveCmd(userId, basicInfo, eduInfo, jobInfo, professionalInfo);
            expertInfoService.saveExpertInfo(expertFullInfoSaveCmd);
        }
    }


}
