package com.ningdatech.pmapi.meeting.controller;


import com.ningdatech.basic.model.IdVo;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import com.ningdatech.pmapi.meeting.entity.dto.MeetingBasicDTO;
import com.ningdatech.pmapi.meeting.entity.req.*;
import com.ningdatech.pmapi.meeting.entity.vo.*;
import com.ningdatech.pmapi.meeting.manage.MeetingManage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * 会议 前端控制器
 * </p>
 *
 * @author WendyYang
 * @since 2022-07-26
 */
@Validated
@RestController
@AllArgsConstructor
@Api(tags = "会议管理")
@RequestMapping("/api/v1/meeting")
public class MeetingController {

    private final MeetingManage meetingManage;

    @PostMapping("/save")
    @ApiOperation("新建会议")
    @WebLog(value = "新建会议")
    public IdVo<Long> meetingCreate(@Valid @RequestBody MeetingBasicDTO po) {
        return meetingManage.save(po);
    }

    @PostMapping("/expertInviteByCreate")
    @ApiOperation("新建会议-专家抽取")
    @WebLog(value = "新建会议-专家抽取")
    public void expertInviteByCreate(@Valid @RequestBody ExpertInviteCreateReq po) {
        meetingManage.expertInviteByCreate(po);
    }

    @ApiOperation("专家抽取数量校验")
    @PostMapping("/expertCountOnChange")
    public ExpertCountOnChangeVO expertCountOnChange(@RequestBody ExpertInviteCreateReq po) {
        return meetingManage.expertCountOnChange(po);
    }

    @ApiOperation("专家抽取员事务列表")
    @GetMapping("/meetingListByManager")
    @WebLog(value = "专家抽取员事务列表")
    public PageVo<MeetingByManagerVO> meetingListByManager(MeetingListReq po) {
        return meetingManage.meetingListByManager(po);
    }

    @ApiOperation("履职记录 | 专家会议列表")
    @GetMapping("/meetingListByExpert")
    @WebLog(value = "履职记录 | 专家会议列表")
    public PageVo<MeetingByManagerVO> meetingListByExpert(MeetingListReq po) {
        return meetingManage.meetingListByExpert(po);
    }

    @ApiOperation("会议详情-基本信息")
    @GetMapping("detail/{meetingId}/basicInfo")
    @WebLog(value = "会议详情-基本信息")
    public MeetingDetailBasicVO meetingBasic(@PathVariable Long meetingId) {
        return meetingManage.getMeetingBasicInfo(meetingId);
    }

    @ApiOperation("会议结果上传")
    @PostMapping("/uploadMeetingResult")
    @WebLog(value = "会议结果上传")
    public void uploadMeetingResult(@Valid @RequestBody MeetingResultReq po) {
        meetingManage.uploadMeetingResult(po);

    }

    @ApiOperation("会议结果详情")
    @GetMapping("/detail/{meetingId}/meetingResult")
    @WebLog(value = "会议结果详情况")
    public MeetingResultVO meetingResultDetail(@PathVariable Long meetingId) {
        return meetingManage.meetingResultDetail(meetingId);
    }

    @ApiOperation("邀请情况详情")
    @GetMapping("/detail/{meetingId}/inviteDetail")
    @WebLog(value = "邀请情况详请")
    public ExpertInviteDetailVO inviteDetail(@PathVariable Long meetingId) {
        return meetingManage.inviteDetail(meetingId);
    }

    @ApiOperation("会议基础信息修改")
    @PostMapping("/basicInfo/modify")
    @WebLog(value = "会议基础信息修改")
    public void meetingBasicInfoModify(@Valid @RequestBody MeetingBasicInfoModifyReq po) {
        meetingManage.meetingBasicInfoModify(po);
    }

    @ApiOperation("会议详情-抽取规则")
    @GetMapping("/detail/inviteRule/{meetingId}")
    @WebLog(value = "会议详情-抽取规则")
    public InviteRuleDetailVO inviteRuleDetail(@PathVariable Long meetingId) {
        return meetingManage.inviteRuleDetail(meetingId);
    }

    @ApiOperation("专家移除")
    @PostMapping("/expertRemove")
    @WebLog(value = "专家移除")
    public void expertRemove(@RequestBody ExpertRemoveReq po) {
        meetingManage.expertRemove(po);
    }

    @ApiOperation("专家替换")
    @PostMapping("/expertReplace")
    @WebLog(value = "专家替换")
    public void expertReplace(@RequestBody ExpertRemoveReq po) {
        meetingManage.expertReplace(po);

    }

    @ApiOperation("确认名单（下发会议通知）")
    @GetMapping("/sendMeetingNotice/{meetingId}")
    @WebLog(value = "确认名单（下发会议通知")
    public void sendMeetingNotice(@PathVariable Long meetingId) {
        meetingManage.sendMeetingNotice(meetingId);

    }

    @ApiOperation("停止抽取")
    @GetMapping("/stopInvite/{meetingId}")
    @WebLog(value = "停止抽取")
    public void stopInvite(@PathVariable Long meetingId) {
        meetingManage.stopRandomInvite(meetingId);

    }

    @ApiOperation("取消会议")
    @PostMapping("/cancelMeeting")
    @WebLog(value = "取消会议")
    public void cancelMeeting(@Valid @RequestBody MeetingCancelReq po) {
        meetingManage.cancelMeeting(po);
    }

    @ApiOperation("邀请函信息")
    @GetMapping("/expertInvitationDetail")
    @WebLog(value = "邀请函信息")
    public ExpertInvitationDetailVO expertInvitationDetail(@RequestParam("meetingId") Long meetingId,
                                                           @RequestParam(required = false) Long expertId) {
        return meetingManage.expertInvitationDetail(meetingId, expertId);
    }

    @ApiOperation("批量补充专家")
    @PostMapping("batchAppointExperts")
    @WebLog(value = "批量补充专家")
    public void batchAppointExperts(@Valid @RequestBody BatchAppointExpertsReq po) {
        meetingManage.batchAppointExperts(po);
    }

    @ApiOperation("确认参加")
    @PostMapping("/confirmAttendByManager")
    @WebLog(value = "确认参加")
    public void confirmAttendByManager(@RequestBody ExpertRemoveReq po) {
        meetingManage.confirmAttendByManager(po);
    }

}
